use anyhow;
use dirs;
use log::warn;
use serde::{Deserialize, Serialize};
use std::{env, path::PathBuf};
use tokio::{fs, io::AsyncReadExt};

use crate::PROGRAM_NAME;

#[derive(Deserialize, Serialize, Debug)]
pub struct Config {
    pub cloudflare_zone_id: Box<str>,
    pub cloudflare_api_key: Box<str>,
}

pub async fn get_config_path() -> Option<PathBuf> {
    match env::current_dir() {
        Ok(current_dir) => {
            let cwd_config = current_dir.join(format!("{}.toml", PROGRAM_NAME));
            if let Ok(meta) = fs::metadata(&cwd_config).await {
                if meta.is_file() {
                    return Some(cwd_config);
                }
            }
        }
        Err(e) => {
            warn!("Failed to get current directory {:?}", &e);
        }
    }

    match dirs::config_dir() {
        Some(config_dir) => {
            let config_file = config_dir.join(PROGRAM_NAME).join("config.toml");
            if let Ok(meta) = fs::metadata(&config_file).await {
                if meta.is_file() {
                    return Some(config_file);
                }
            }
        }
        None => {}
    }

    None
}

pub async fn read_config(path: &PathBuf) -> anyhow::Result<Config> {
    let mut file = fs::File::open(&path).await?;
    let mut buf = String::new();
    file.read_to_string(&mut buf).await?;
    Ok(toml::from_str(&buf)?)
}
